use crate::{AccountId, Amount, Status, TransactionId};
use serde::{Serialize, Serializer};

#[derive(Clone, Debug, Eq, Ord, PartialOrd, PartialEq)]
pub struct PaymentId {
    pub transaction: TransactionId,
    pub position: usize,
}

impl Serialize for PaymentId {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer,
    {
        let serialized = self.to_string();
        serializer.serialize_str(&serialized)
    }
}

impl ToString for PaymentId {
    fn to_string(&self) -> String {
        format!("{}:{}", self.transaction, self.position)
    }
}

#[derive(Clone, Debug, Eq, PartialEq, Serialize)]
pub struct Payment {
    pub id: PaymentId,
    pub to: AccountId,
    pub amount: Amount,
    #[serde(skip)]
    pub status: Status,
    #[serde(skip)]
    pub spent_by: Option<TransactionId>,
}

impl Payment {
    pub fn is_spendable(&self) -> bool {
        self.spent_by.is_none() && self.status == Status::Settled
    }
}
