use crate::{storage, Amount, Asset, Status, TransactionId};

#[derive(thiserror::Error, Debug)]
pub enum Error {
    #[error("Payment {0} is already spent")]
    SpentPayment(usize),

    #[error("Payment {0} is in the incorrect status {1}")]
    InvalidPaymentStatus(usize, Status),

    #[error("Spending {0:?} and issuing {1:?}. Both amounts must match")]
    InvalidAmount(Amount, Amount),

    #[error("Missing input payment for asset {0}")]
    MissingSpendingAsset(Asset),

    #[error("Missing output payment for asset {0}")]
    MissingPaymentAsset(Asset),

    #[error("Transaction updates are not allowed")]
    TransactionUpdatesNotAllowed,

    #[error("Mismatch between spend and create payment assets")]
    MismatchSpendingPayingAssets,

    #[error("Status transition from {0} to {1} is not allowed")]
    StatusTransitionNotAllowed(Status, Status),

    #[error("Storage: {0}")]
    Storage(#[from] storage::Error),

    #[error("Internal error at serializing: {0}")]
    Internal(#[from] Box<bincode::ErrorKind>),

    #[error("Invalid calculated id {0} (expected {1})")]
    InvalidTransactionId(TransactionId, TransactionId),

    #[error("Overflow")]
    Overflow,
}
