use serde::{Deserialize, Serialize};

#[derive(thiserror::Error, Debug)]
pub enum Error {
    #[error("Invalid status: {0}")]
    InvalidStatus(u32),
}

#[derive(Debug, Clone, Copy, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "snake_case")]
pub enum Type {
    Deposit,
    Withdrawal,
    Transaction,
    Internal,
}

impl Type {
    #[inline]
    pub fn is_transaction(&self) -> bool {
        matches!(self, Self::Transaction | Self::Internal)
    }
}

impl TryFrom<u32> for Type {
    type Error = Error;
    fn try_from(value: u32) -> Result<Self, Self::Error> {
        match value {
            0 => Ok(Self::Transaction),
            1 => Ok(Self::Deposit),
            2 => Ok(Self::Withdrawal),
            1000 => Ok(Self::Internal),
            _ => Err(Error::InvalidStatus(value)),
        }
    }
}

impl From<Type> for u32 {
    fn from(value: Type) -> Self {
        (&value).into()
    }
}

impl From<&Type> for u32 {
    fn from(value: &Type) -> Self {
        match value {
            Type::Transaction => 0,
            Type::Deposit => 1,
            Type::Withdrawal => 2,
            Type::Internal => 1000,
        }
    }
}
